import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import '../../../config/app_colors.dart';
import '../../../models/counts_by_day.dart';
import '../../../utils/color_extensions.dart';

class CountsByDayCardContentChart extends StatelessWidget {
  final CountsByDayData chartData;

  const CountsByDayCardContentChart({super.key, required this.chartData});

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 100.0,
      child: LayoutBuilder(builder: (context, constraints) {
        final double maxWidth = constraints.maxWidth;
        final int barsCount = this.chartData.data.keys.length;
        final double barWidth = 0.7 * (maxWidth / barsCount);

        return BarChart(
          BarChartData(
            barGroups: getDataCounts(barWidth),
            backgroundColor: Theme.of(context).colorScheme.onBackground,
            borderData: getBorderData(),
            gridData: getGridData(),
            titlesData: getTitlesData(),
            barTouchData: getBarTouchData(),
            maxY: getNextRoundNumber(getMaxCountsValue(), 10),
          ),
        );
      }),
    );
  }

  double getMaxCountsValue() {
    double maxValue = 0;

    this.chartData.data.keys.forEach((key) {
      double? counts = this.chartData.data[key];
      if (counts != null) {
        if (counts > maxValue) {
          maxValue = counts;
        }
      }
    });

    return maxValue;
  }

  double getNextRoundNumber(double number, int scale) {
    return scale * ((number ~/ scale).toInt() + 1);
  }

  List<BarChartGroupData> getDataCounts(double barWidth) {
    List<BarChartGroupData> data = [];

    this.chartData.data.keys.forEach((day) {
      final double? counts = this.chartData.data[day];

      if (counts != null) {
        Color barColor = AppColors.contentColorBlack;
        switch (day) {
          case 1:
            barColor = Color.fromARGB(255, 255, 99, 132);
            break;
          case 2:
            barColor = Color.fromARGB(255, 255, 159, 64);
            break;
          case 3:
            barColor = Color.fromARGB(255, 255, 205, 86);
            break;
          case 4:
            barColor = Color.fromARGB(255, 75, 192, 192);
            break;
          case 5:
            barColor = Color.fromARGB(255, 54, 162, 235);
            break;
          case 6:
            barColor = Color.fromARGB(255, 153, 102, 255);
            break;
          case 7:
            barColor = Color.fromARGB(255, 201, 203, 207);
            break;
          default:
        }

        data.add(BarChartGroupData(
          x: day,
          barRods: [
            BarChartRodData(
              toY: counts,
              color: barColor,
              width: barWidth,
              borderRadius: BorderRadius.all(Radius.zero),
              borderSide: BorderSide(
                color: barColor.darken(20),
              ),
            ),
          ],
        ));
      }
    });

    return data;
  }

  FlBorderData getBorderData() {
    return FlBorderData(
      show: true,
      border: Border.all(
        color: AppColors.borderColor,
        width: 2,
      ),
    );
  }

  FlGridData getGridData() {
    return const FlGridData(
      show: true,
    );
  }

  FlTitlesData getTitlesData() {
    const AxisTitles none = const AxisTitles(
      sideTitles: SideTitles(showTitles: false),
    );

    final AxisTitles verticalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 30,
        getTitlesWidget: getVerticalTitlesWidget,
        interval: 10,
      ),
    );
    final AxisTitles horizontalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 20,
        getTitlesWidget: getHorizontalTitlesWidget,
      ),
    );

    return FlTitlesData(
      show: true,
      bottomTitles: horizontalTitles,
      leftTitles: none,
      topTitles: none,
      rightTitles: verticalTitles,
    );
  }

  Widget getVerticalTitlesWidget(double value, TitleMeta meta) {
    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 4,
      child: Text(
        value.toInt().toString(),
        style: const TextStyle(
          color: AppColors.mainTextColor1,
          fontSize: 12,
        ),
      ),
    );
  }

  Widget getHorizontalTitlesWidget(double value, TitleMeta meta) {
    final int day = value.toInt();
    String dayShortName = '';
    switch (day) {
      case 1:
        dayShortName = 'MON';
        break;
      case 2:
        dayShortName = 'TUE';
        break;
      case 3:
        dayShortName = 'WED';
        break;
      case 4:
        dayShortName = 'THU';
        break;
      case 5:
        dayShortName = 'FRI';
        break;
      case 6:
        dayShortName = 'SAT';
        break;
      case 7:
        dayShortName = 'SUN';
        break;
      default:
    }

    final String text = tr(dayShortName);

    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 2,
      child: Text(
        text,
        style: const TextStyle(
          color: AppColors.mainTextColor1,
          fontSize: 11,
        ),
      ),
    );
  }

  BarTouchData getBarTouchData() {
    return BarTouchData(
      enabled: true,
      touchTooltipData: BarTouchTooltipData(
        tooltipBgColor: Colors.transparent,
        tooltipPadding: EdgeInsets.zero,
        tooltipMargin: 2,
        getTooltipItem: (
          BarChartGroupData group,
          int groupIndex,
          BarChartRodData rod,
          int rodIndex,
        ) {
          return BarTooltipItem(
            rod.toY.round().toString(),
            const TextStyle(
              color: AppColors.mainTextColor2,
              fontWeight: FontWeight.bold,
              fontSize: 10,
            ),
          );
        },
      ),
    );
  }
}
